@@ -1,17 +1,15 @@
-import math
 import os
 from six import StringIO
-from six.moves import queue as Queue
 import sys
 import time
 import threading
-import mimetypes
 
 from dateutil.parser import parse
 from dateutil.tz import tzlocal
 
 from botocore.compat import quote
-from awscli.customizations.s3.tasks import UploadPartTask, DownloadPartTask
+from botocore import xform_name
+from awscli.customizations.s3.tasks import DownloadPartTask
 from awscli.customizations.s3.utils import find_bucket_key, MultiCounter, \
     retrieve_http_etag, check_etag, check_error, operate, NoBlockQueue, \
     uni_print, guess_content_type
@@ -198,7 +196,7 @@ class FileInfo(TaskInfo):
     """
     This is a child object of the ``TaskInfo`` object.  It can perform more
     operations such as ``upload``, ``download``, ``copy``, ``delete``,
-    ``move``, `multi_upload``, and ``multi_download``.  Similiarly to
+    ``move``, and ``multi_download``.  Similiarly to
     ``TaskInfo`` objects attributes like ``session`` need to be set in order
     to perform operations. Multipart operations need to run ``set_multi`` in
     order to be able to run.
@@ -232,51 +230,91 @@ class FileInfo(TaskInfo):
         if parameters:
             self.parameters = parameters
         else:
-            self.parameters = {'acl': None}
+            self.parameters = {'acl': None,
+                               'sse': None}
 
         # Required for multipart uploads and downloads.  Use ``set_multi``
         # function to set these.
         self.executer = None
-        self.printQueue = None
+        self.print_queue = None
         self.is_multi = False
         self.interrupt = None
         self.chunksize = None
 
-    def set_multi(self, executer, printQueue, interrupt, chunksize):
+    def set_multi(self, executer, print_queue, interrupt, chunksize):
         """
         This sets all of the necessary attributes to perform a multipart
         operation.
         """
         self.executer = executer
-        self.printQueue = printQueue
+        self.print_queue = print_queue
         self.is_multi = True
         self.interrupt = interrupt
         self.chunksize = chunksize
 
+    def _permission_to_param(self, permission):
+        if permission == 'read':
+            return 'grant_read'
+        if permission == 'full':
+            return 'grant_full_control'
+        if permission == 'readacl':
+            return 'grant_read_acp'
+        if permission == 'writeacl':
+            return 'grant_write_acp'
+        raise ValueError('permission must be one of: '
+                         'read|readacl|writeacl|full')
+
+    def _handle_object_params(self, params):
+        if self.parameters['acl']:
+            params['acl'] = self.parameters['acl'][0]
+        if self.parameters['grants']:
+            for grant in self.parameters['grants']:
+                try:
+                    permission, grantee = grant.split('=', 1)
+                except ValueError:
+                    raise ValueError('grants should be of the form '
+                                     'permission=principal')
+                params[self._permission_to_param(permission)] = grantee
+        if self.parameters['sse']:
+            params['server_side_encryption'] = 'AES256'
+        if self.parameters['storage_class']:
+            params['storage_class'] = self.parameters['storage_class'][0]
+        if self.parameters['website_redirect']:
+            params['website_redirect_location'] = self.parameters['website_redirect'][0]
+        if self.parameters['guess_mime_type']:
+            self._inject_content_type(params, self.src)
+        if self.parameters['content_type']:
+            params['content_type'] = self.parameters['content_type'][0]
+        if self.parameters['cache_control']:
+            params['cache_control'] = self.parameters['cache_control'][0]
+        if self.parameters['content_disposition']:
+            params['content_disposition'] = self.parameters['content_disposition'][0]
+        if self.parameters['content_encoding']:
+            params['content_encoding'] = self.parameters['content_encoding'][0]
+        if self.parameters['content_language']:
+            params['content_language'] = self.parameters['content_language'][0]
+        if self.parameters['expires']:
+            params['expires'] = self.parameters['expires'][0]
+
     def upload(self):
         """
         Redirects the file to the multipart upload function if the file is
         large.  If it is small enough, it puts the file as an object in s3.
         """
-        if not self.is_multi:
-            body = read_file(self.src)
-            bucket, key = find_bucket_key(self.dest)
-            if sys.version_info[:2] == (2, 6):
-                stream_body = StringIO(body)
-            else:
-                stream_body = bytearray(body)
-            params = {'endpoint': self.endpoint, 'bucket': bucket, 'key': key}
-            if body:
-                params['body'] = stream_body
-            if self.parameters['acl']:
-                params['acl'] = self.parameters['acl'][0]
-            if self.parameters['guess_mime_type']:
-                self._inject_content_type(params, self.src)
-            response_data, http = operate(self.service, 'PutObject', params)
-            etag = retrieve_http_etag(http)
-            check_etag(etag, body)
+        body = read_file(self.src)
+        bucket, key = find_bucket_key(self.dest)
+        if sys.version_info[:2] == (2, 6):
+            stream_body = StringIO(body)
         else:
-            self.multi_upload()
+            stream_body = bytearray(body)
+        params = {'endpoint': self.endpoint, 'bucket': bucket, 'key': key}
+        if body:
+            params['body'] = stream_body
+        self._handle_object_params(params)
+        response_data, http = operate(self.service, 'PutObject', params)
+        etag = retrieve_http_etag(http)
+        check_etag(etag, body)
+
 
     def _inject_content_type(self, params, filename):
         # Add a content type param if we can guess the type.
@@ -301,12 +339,11 @@ class FileInfo(TaskInfo):
         """
         Copies a object in s3 to another location in s3.
         """
-        copy_source = quote(self.src)
+        copy_source = quote(self.src.encode('utf-8'), safe='/~')
         bucket, key = find_bucket_key(self.dest)
         params = {'endpoint': self.endpoint, 'bucket': bucket,
                   'copy_source': copy_source, 'key': key}
-        if self.parameters['acl']:
-            params['acl'] = self.parameters['acl'][0]
+        self._handle_object_params(params)
         response_data, http = operate(self.service, 'CopyObject', params)
 
     def delete(self):
@@ -338,66 +375,14 @@ class FileInfo(TaskInfo):
             raise Exception("Invalid path arguments for mv")
         self.delete()
 
-    def multi_upload(self):
-        """
-        Performs multipart uploads.  It initiates the multipart upload.
-        It creates a queue ``part_queue`` which is directly responsible
-        with controlling the progress of the multipart upload.  It then
-        creates ``UploadPartTasks`` for threads to run via the
-        ``executer``.  This fucntion waits for all of the parts in the
-        multipart upload to finish, and then it completes the multipart
-        upload.  This method waits on its parts to finish.  So, threads
-        are required to process the parts for this function to complete.
-        """
-        part_queue = NoBlockQueue(self.interrupt)
-        complete_upload_queue = Queue.PriorityQueue()
-        part_counter = MultiCounter()
-        counter_lock = threading.Lock()
+    def create_multipart_upload(self):
         bucket, key = find_bucket_key(self.dest)
         params = {'endpoint': self.endpoint, 'bucket': bucket, 'key': key}
-        if self.parameters['acl']:
-            params['acl'] = self.parameters['acl'][0]
-        if self.parameters['guess_mime_type']:
-            self._inject_content_type(params, self.src)
+        self._handle_object_params(params)
         response_data, http = operate(self.service, 'CreateMultipartUpload',
                                       params)
         upload_id = response_data['UploadId']
-        size_uploads = self.chunksize
-        num_uploads = int(math.ceil(self.size/float(size_uploads)))
-        for i in range(1, (num_uploads + 1)):
-            part_info = (self, upload_id, i, size_uploads)
-            part_queue.put(part_info)
-            task = UploadPartTask(session=self.session, executer=self.executer,
-                                  part_queue=part_queue,
-                                  dest_queue=complete_upload_queue,
-                                  region=self.region,
-                                  printQueue=self.printQueue,
-                                  interrupt=self.interrupt,
-                                  part_counter=part_counter,
-                                  counter_lock=counter_lock)
-            self.executer.submit(task)
-        part_queue.join()
-        # The following ensures that if the multipart upload is in progress,
-        # all part uploads finish before aborting or completing.  This
-        # really only applies when an interrupt signal is sent because the
-        # ``part_queue.join()`` ensures this if the process is not
-        # interrupted.
-        while part_counter.count:
-            time.sleep(0.1)
-        parts_list = []
-        while not complete_upload_queue.empty():
-            part = complete_upload_queue.get()
-            parts_list.append(part[1])
-        if len(parts_list) == num_uploads:
-            parts = {'Parts': parts_list}
-            params = {'endpoint': self.endpoint, 'bucket': bucket, 'key': key,
-                      'upload_id': upload_id, 'multipart_upload': parts}
-            operate(self.service, 'CompleteMultipartUpload', params)
-        else:
-            abort_params = {'endpoint': self.endpoint, 'bucket': bucket,
-                            'key': key, 'upload_id': upload_id}
-            operate(self.service, 'AbortMultipartUpload', abort_params)
-            raise Exception()
+        return upload_id
 
     def multi_download(self):
         """
@@ -434,7 +419,7 @@ class FileInfo(TaskInfo):
                                         part_queue=part_queue,
                                         dest_queue=dest_queue,
                                         f=f, region=self.region,
-                                        printQueue=self.printQueue,
+                                        print_queue=self.print_queue,
                                         write_lock=write_lock,
                                         part_counter=part_counter,
                                         counter_lock=counter_lock)
@@ -456,3 +441,4 @@ class FileInfo(TaskInfo):
         last_update_tuple = self.last_update.timetuple()
         mod_timestamp = time.mktime(last_update_tuple)
         os.utime(self.dest, (int(mod_timestamp), int(mod_timestamp)))
+
